==> 0.18.2.tar.gz: lib/components/senior_backdrop/senior_backdrop_widget.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_backdrop_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorBackdrop extends StatefulWidget {
  const SeniorBackdrop({
    Key? key,
    this.actions,
    this.backdropTabBarInfo,
    required this.body,
    this.centerTitle,
    this.hasTopPadding = true,
    this.hideLeading = false,
    this.leading,
    this.onRefresh,
    this.onTapBack,
    this.systemOverlayStyle,
    this.style,
    required this.title,
  }) : super(key: key);

  final List<Widget>? actions;
  final SeniorBackdropTabBarInfo? backdropTabBarInfo;
  final Widget body;
  final bool? centerTitle;
  final bool hasTopPadding;
  final bool hideLeading;
  final Widget? leading;
  final RefreshCallback? onRefresh;
  final Function()? onTapBack;
  final SystemUiOverlayStyle? systemOverlayStyle;
  final SeniorBackdropStyle? style;
  final Widget title;

  @override
  State<SeniorBackdrop> createState() => _SeniorBackdropState();
}

class _SeniorBackdropState extends State<SeniorBackdrop> {
  final ScrollController backdropController = ScrollController(keepScrollOffset: false);
  final appBarHeight = 56;
  final tabBarHeight = 48;
  bool hasPadding = false;
  double appBarCurrentHeight = 0.0;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Container(
      height: MediaQuery.of(context).size.height,
      decoration: _buildBoxDecoration(theme, defaultTheme),
      child: Scaffold(
        backgroundColor: Colors.transparent,
        body: SafeArea(
          child: NotificationListener<ScrollNotification>(
            onNotification: (notify) {
              //This delayed is necessary because of the refreshIndicator used in the body.
              //When using a setState in a build process it gives a problem
              //keep the duration as zero ALWAYS.
              Future.delayed(Duration.zero, () {
                setStateIfMounted(() {
                  if (widget.backdropTabBarInfo == null) _updateAppBarCurrentHeight();
                  if (notify is ScrollUpdateNotification) {
                    if (widget.backdropTabBarInfo != null) _updateAppBarCurrentHeight();
                    hasPadding = _hasPadding(
                      appBarCurrentHeight,
                      backdropController.position.userScrollDirection,
                    );
                  }
                });
              });

              return true;
            },
            child: NestedScrollView(
              controller: backdropController,
              floatHeaderSlivers: true,
              physics: PageScrollPhysics(),
              headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
                return <Widget>[
                  _buildAppBar(theme, defaultTheme),
                ];
              },
              body: _buildBody(theme, defaultTheme),
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildBody(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Padding(
      padding: EdgeInsets.only(top: (hasPadding) ? _getPadding(widget.backdropTabBarInfo == null) : 0),
      child: Container(
        padding: EdgeInsets.only(
          top: widget.hasTopPadding ? SeniorRadius.huge : 0,
        ),
        decoration: BoxDecoration(
          color: widget.style?.bodyColor ??
              theme?.backdropTheme?.style?.bodyColor ??
              defaultTheme.backdropTheme?.style?.bodyColor ??
              SeniorColors.pureWhite,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(SeniorRadius.huge),
            topRight: Radius.circular(SeniorRadius.huge),
          ),
        ),
        child: RefreshIndicator(
          color: SeniorColors.primaryColor,
          onRefresh: widget.onRefresh != null ? widget.onRefresh! : () async {},
          child: widget.body,
        ),
      ),
    );
  }

  SliverAppBar _buildAppBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return SliverAppBar(
      pinned: true,
      primary: true,
      floating: true,
      elevation: 0,
      backgroundColor: Colors.transparent,
      shadowColor: Colors.transparent,
      actions: widget.actions,
      title: widget.title,
      centerTitle: widget.centerTitle,
      bottom: widget.backdropTabBarInfo != null ? _buildTabBar(theme, defaultTheme) : null,
      systemOverlayStyle: widget.systemOverlayStyle,
      leadingWidth: widget.hideLeading ? 0 : null,
      leading: widget.hideLeading
          ? SizedBox.shrink()
          : widget.leading == null
              ? IconButton(
                  icon: const Icon(FontAwesomeIcons.angleLeft),
                  onPressed: widget.onTapBack ??
                      () {
                        Navigator.pop(context);
                      },
                )
              : widget.leading,
    );
  }

  TabBar _buildTabBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final Color labelColor = widget.style?.selectedLabelColor ??
        theme?.backdropTheme?.style?.selectedLabelColor ??
        defaultTheme.backdropTheme?.style?.selectedLabelColor ??
        SeniorColors.pureWhite;

    final Color unselectedLabelColor = widget.style?.unselectedLabelColor ??
        theme?.backdropTheme?.style?.unselectedLabelColor ??
        defaultTheme.backdropTheme?.style?.unselectedLabelColor ??
        SeniorColors.pureWhite.withOpacity(.6);

    final Color indicatorColor = widget.style?.tabIndicatorColor ??
        theme?.backdropTheme?.style?.tabIndicatorColor ??
        defaultTheme.backdropTheme?.style?.tabIndicatorColor ??
        SeniorColors.pureWhite;

    return TabBar(
      tabs: widget.backdropTabBarInfo!.tabs.map((tab) => Tab(text: tab)).toList(),
      labelColor: labelColor,
      unselectedLabelColor: unselectedLabelColor,
      labelStyle: SeniorTypography.bodySemiBold(color: labelColor),
      indicator: SeniorTabIndicator(color: indicatorColor),
      physics: BouncingScrollPhysics(),
      isScrollable: widget.backdropTabBarInfo!.isScrollable,
      onTap: widget.backdropTabBarInfo!.onTap,
      controller: widget.backdropTabBarInfo!.controller,
    );
  }

  BoxDecoration _buildBoxDecoration(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return BoxDecoration(
      gradient: LinearGradient(
        colors: widget.style?.headerColors ??
            theme?.backdropTheme?.style?.headerColors ??
            defaultTheme.backdropTheme?.style?.headerColors ??
            SeniorColors.primaryGradientColors,
      ),
    );
  }

  void setStateIfMounted(function) {
    if (mounted) setState(function);
  }

  double _getPadding(bool noHasBottomBar) =>
      noHasBottomBar ? (appBarHeight - appBarCurrentHeight) : (tabBarHeight - appBarCurrentHeight);

  bool _hasPadding(double appBarScrollPosition, ScrollDirection scrollDirection) => (widget.backdropTabBarInfo == null)
      ? (appBarScrollPosition < appBarHeight)
      : (appBarScrollPosition < tabBarHeight);

  double _updateAppBarCurrentHeight() => appBarCurrentHeight = backdropController.position.extentAfter;
}

class SeniorTabIndicator extends Decoration {
  final BoxPainter _painter;
  final Color color;

  SeniorTabIndicator({required this.color}) : _painter = _SeniorPainter(color);

  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) => _painter;
}

class _SeniorPainter extends BoxPainter {
  final Paint _paint;

  _SeniorPainter(Color color)
      : _paint = Paint()
          ..color = color
          ..isAntiAlias = true;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration cfg) {
    final Rect rect = Offset(offset.dx + cfg.size!.width / 2 - 12, (cfg.size!.height - 10)) & Size(24, 2);
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        rect,
        bottomLeft: Radius.circular(1),
        bottomRight: Radius.circular(1),
        topLeft: Radius.circular(1),
        topRight: Radius.circular(1),
      ),
      _paint,
    );
  }
}

class SeniorBackdropTabBarInfo {
  final bool isScrollable;
  final Function(int) onTap;
  final List<String> tabs;
  final int initialIndex;
  final TabController controller;

  SeniorBackdropTabBarInfo({
    this.isScrollable = false,
    required this.onTap,
    required this.tabs,
    required this.initialIndex,
    required this.controller,
  });
}

==> 0.19.8.tar.gz: lib/components/senior_backdrop/senior_backdrop_widget.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_backdrop_style.dart';
import '../../themes/senior_theme_provider.dart';
import '../../themes/senior_theme.dart';

class SeniorBackdrop extends StatefulWidget {
  /// Creates the SDS Backdrop component.
  ///
  /// The [body] and [title] parameters are required.
  const SeniorBackdrop({
    Key? key,
    this.actions,
    this.backdropTabBarInfo,
    required this.body,
    this.centerTitle,
    this.hasTopPadding = true,
    this.hideLeading = false,
    this.leading,
    this.onRefresh,
    this.onTapBack,
    this.systemOverlayStyle,
    this.style,
    required this.title,
  }) : super(key: key);

  /// The value that will be passed to the [SliverAppBar.actions] parameter.
  /// A list of Widgets to display in a row after the title widget.
  final List<Widget>? actions;

  /// Tab settings for the backdrop.
  final SeniorBackdropTabBarInfo? backdropTabBarInfo;

  /// The content displayed on the backdrop.
  /// The body is displayed on the front layer.
  final Widget body;

  /// The value that will be passed to the [SliverAppBar.centerTitle] parameter.
  /// Whether the title should be centered.
  final bool? centerTitle;

  /// Defines whether to add a padding to the top of the front layer.
  ///
  /// The default value is true.
  final bool hasTopPadding;

  /// Defines whether leading will be omitted.
  ///
  /// The default value is false.
  final bool hideLeading;

  /// The value that will be passed to the [SliverAppBar.leading] parameter.
  /// A widget to display before the toolbar's title.
  final Widget? leading;

  /// Event of reloading the view in a RefreshIndicator.
  /// When this parameter is null, a RefreshIndicator is not created.
  final RefreshCallback? onRefresh;

  /// Event fired when leading returns is tapped.
  /// The return leading is created when the value of the [leading] parameter is null.
  final Function()? onTapBack;

  /// The value that will be passed to the [SliverAppBar.systemOverlayStyle] parameter.
  /// Specifies the style to use for the system overlays that overlap the AppBar.
  final SystemUiOverlayStyle? systemOverlayStyle;

  /// The component's style definitions.
  /// Allows you to configure:
  /// [SeniorBackdropStyle.bodyColor] the body color of the backdrop.
  /// [SeniorBackdropStyle.headerColors] the color of the backdrop header. Component top.
  /// [SeniorBackdropStyle.selectedLabelColor] the color of the selected labels.
  /// [SeniorBackdropStyle.tabIndicatorColor] the indicator color of the selected tab on the backdrop.
  /// [SeniorBackdropStyle.unselectedLabelColor] the color of unselected labels.
  final SeniorBackdropStyle? style;

  /// The value that will be passed to the [SliverAppBar.title] parameter.
  /// The primary widget displayed in the app bar.
  final Widget title;

  @override
  State<SeniorBackdrop> createState() => _SeniorBackdropState();
}

class _SeniorBackdropState extends State<SeniorBackdrop> {
  final ScrollController backdropController = ScrollController(keepScrollOffset: false);
  final double appBarHeight = 56.0;
  final double tabBarHeight = 48.0;
  bool hasPadding = false;
  double appBarCurrentHeight = 0.0;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    return Container(
      height: MediaQuery.of(context).size.height,
      decoration: _buildBoxDecoration(theme, defaultTheme),
      child: Scaffold(
        backgroundColor: Colors.transparent,
        body: SafeArea(
          child: NotificationListener<ScrollNotification>(
            onNotification: (notify) {
              //This delayed is necessary because of the refreshIndicator used in the body.
              //When using a setState in a build process it gives a problem
              //keep the duration as zero ALWAYS.
              Future.delayed(Duration.zero, () {
                setStateIfMounted(() {
                  if (widget.backdropTabBarInfo == null) {
                    _updateAppBarCurrentHeight();
                  }
                  if (notify is ScrollUpdateNotification) {
                    if (widget.backdropTabBarInfo != null) {
                      _updateAppBarCurrentHeight();
                    }
                    hasPadding = _hasPadding(
                      appBarCurrentHeight,
                      backdropController.position.userScrollDirection,
                    );
                  }
                });
              });

              return true;
            },
            child: NestedScrollView(
              controller: backdropController,
              floatHeaderSlivers: true,
              physics: PageScrollPhysics(),
              headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
                return <Widget>[
                  _buildAppBar(theme, defaultTheme),
                ];
              },
              body: _buildBody(theme, defaultTheme),
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildBody(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final borderRadius = BorderRadius.vertical(
      top: Radius.circular(SeniorRadius.huge),
    );

    final body = ClipRRect(
      borderRadius: borderRadius,
      child: widget.body,
    );

    return Padding(
      padding: EdgeInsets.only(top: (hasPadding) ? _getPadding(widget.backdropTabBarInfo == null) : 0),
      child: Container(
        padding: EdgeInsets.only(
          top: widget.hasTopPadding ? SeniorRadius.huge : 0,
        ),
        decoration: BoxDecoration(
          color: widget.style?.bodyColor ??
              theme?.backdropTheme?.style?.bodyColor ??
              defaultTheme.backdropTheme?.style?.bodyColor ??
              SeniorColors.pureWhite,
          borderRadius: borderRadius,
        ),
        child: widget.onRefresh != null
            ? RefreshIndicator(
                color: SeniorColors.primaryColor,
                onRefresh: widget.onRefresh!,
                child: body,
              )
            : body,
      ),
    );
  }

  SliverAppBar _buildAppBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return SliverAppBar(
      pinned: true,
      primary: true,
      floating: true,
      elevation: 0,
      backgroundColor: Colors.transparent,
      shadowColor: Colors.transparent,
      actions: widget.actions,
      title: widget.title,
      centerTitle: widget.centerTitle,
      bottom: widget.backdropTabBarInfo != null ? _buildTabBar(theme, defaultTheme) : null,
      systemOverlayStyle: widget.systemOverlayStyle,
      leadingWidth: widget.hideLeading ? 0 : null,
      leading: widget.hideLeading
          ? SizedBox.shrink()
          : widget.leading == null
              ? IconButton(
                  icon: const Icon(FontAwesomeIcons.angleLeft),
                  onPressed: widget.onTapBack ??
                      () {
                        Navigator.pop(context);
                      },
                )
              : widget.leading,
    );
  }

  TabBar _buildTabBar(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final Color labelColor = widget.style?.selectedLabelColor ??
        theme?.backdropTheme?.style?.selectedLabelColor ??
        defaultTheme.backdropTheme?.style?.selectedLabelColor ??
        SeniorColors.pureWhite;

    final Color unselectedLabelColor = widget.style?.unselectedLabelColor ??
        theme?.backdropTheme?.style?.unselectedLabelColor ??
        defaultTheme.backdropTheme?.style?.unselectedLabelColor ??
        SeniorColors.pureWhite.withOpacity(.6);

    final Color indicatorColor = widget.style?.tabIndicatorColor ??
        theme?.backdropTheme?.style?.tabIndicatorColor ??
        defaultTheme.backdropTheme?.style?.tabIndicatorColor ??
        SeniorColors.pureWhite;

    return TabBar(
      tabs: widget.backdropTabBarInfo!.tabs.map((tab) => Tab(text: tab)).toList(),
      labelColor: labelColor,
      unselectedLabelColor: unselectedLabelColor,
      labelStyle: SeniorTypography.bodySemiBold(color: labelColor),
      indicator: SeniorTabIndicator(color: indicatorColor),
      physics: BouncingScrollPhysics(),
      isScrollable: widget.backdropTabBarInfo!.isScrollable,
      onTap: widget.backdropTabBarInfo!.onTap,
      controller: widget.backdropTabBarInfo!.controller,
    );
  }

  BoxDecoration _buildBoxDecoration(SeniorTheme? theme, SeniorTheme defaultTheme) {
    return BoxDecoration(
      gradient: LinearGradient(
        colors: widget.style?.headerColors ??
            theme?.backdropTheme?.style?.headerColors ??
            defaultTheme.backdropTheme?.style?.headerColors ??
            SeniorColors.primaryGradientColors,
      ),
    );
  }

  void setStateIfMounted(function) {
    if (mounted) setState(function);
  }

  double _getPadding(bool noHasBottomBar) {
    return noHasBottomBar ? (appBarHeight - appBarCurrentHeight) : (tabBarHeight - appBarCurrentHeight);
  }

  bool _hasPadding(double appBarScrollPosition, ScrollDirection scrollDirection) {
    return (widget.backdropTabBarInfo == null)
        ? (appBarScrollPosition < appBarHeight)
        : (appBarScrollPosition < tabBarHeight);
  }

  double _updateAppBarCurrentHeight() => appBarCurrentHeight = backdropController.position.extentAfter;
}

class SeniorTabIndicator extends Decoration {
  final BoxPainter _painter;
  final Color color;

  SeniorTabIndicator({required this.color}) : _painter = _SeniorPainter(color);

  @override
  BoxPainter createBoxPainter([VoidCallback? onChanged]) => _painter;
}

class _SeniorPainter extends BoxPainter {
  final Paint _paint;

  _SeniorPainter(Color color)
      : _paint = Paint()
          ..color = color
          ..isAntiAlias = true;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration cfg) {
    final Rect rect = Offset(offset.dx + cfg.size!.width / 2 - 12, (cfg.size!.height - 10)) & Size(24, 2);
    canvas.drawRRect(
      RRect.fromRectAndCorners(
        rect,
        bottomLeft: Radius.circular(1),
        bottomRight: Radius.circular(1),
        topLeft: Radius.circular(1),
        topRight: Radius.circular(1),
      ),
      _paint,
    );
  }
}

class SeniorBackdropTabBarInfo {
  /// Configurações para adicionar abas no backdrop.
  /// Os parâmetros [controller], [initialIndex], [onTap] e [tabs] são obrigatórios.
  SeniorBackdropTabBarInfo({
    required this.controller,
    required this.initialIndex,
    this.isScrollable = false,
    required this.onTap,
    required this.tabs,
  });

  ///
  final TabController controller;

  ///
  final int initialIndex;

  ///
  final bool isScrollable;

  ///
  final Function(int) onTap;

  ///
  final List<String> tabs;
}
